// Declares common types used in FreePaTris

unit Types;

interface

uses 
	Crt,
	SysUtils;

type
	
	// Enumeration of gamestate Ids
	TGamestateId = (StateGame, StateMenu, StateQuit);
	
	// Enumeration of block types
	TBlockId = (TBlockI := char('I'), TBlockJ := char('J'), TBlockL := char('L'),
                TBlockO := char('O'), TBlockS := char('S'), TBlockT := char('T'),
                TBlockZ := char('Z'));
	
	// Orientations clockwise
	TBlockOrientation = (Up, Right, Down, Left);
	
	// Represents boundaries
	TBoundary = record
		Upper, Left, Lower, Right : byte;
	end; 
	
	// Three dimensional array of block cells
	// [Orientation, X, Y]
	TBlockCells = array of array of array of boolean;
	
	// Represents a falling block
	TBlock = record
		// The position of a block
		X,Y : shortint;
		// The colour of the block
		Colour : byte;
		// The cells of this block
		Cells : TBlockCells;
		// The id of this block
		Id : TBlockId;
		// The current orientation of the block
		Orientation : shortint;
		// The boundaries of this block
		Boundary : TBoundary;
	end;
	
	// Represents a single cell of the gamefield
	TGameFieldCell = record
		// True if the cell is blocked
		Blocked : boolean;
		
		// The colour of this cell
		Colour : byte;
	end;
	
	// two dimensional array of gamefield cells
	TGameField = array of array of TGameFieldCell;

	// Represents a running game
	TGameState = record
    	Gamefield : TGameField; // The gamefield
		TimeTillDraw : comp; // Time till next redraw
		Block : TBlock;  // The currently falling block
		TimeTillFall : comp; // Time till the block needs to be moved down
		RowsFilled : word; // The number of rows deleted so far
		NextBlock : TBlockId; // The coming block
	end;
	
const
	// The gamescreen will be redrawn after this time (in ms), this should
	// avoid to heavy flickering
	// NOTICE: The falltime will never fall below this value
	TargetFrametime = 200;
	
	// The default time a block needs to fall
	DefaultFallTime = 700;
	// The time that is subtracted from the fall time each level
	DefaultFallTimeStep = 50;
	// Raise speed every n rows
	DefaultRowLevelStep = 10;
	
	// The width and height of the gamefield
	GamefieldWidth = 10;
	GamefieldHeight = 20;
	
	// The background colour of the gamefield
	GamefieldBackground = Black;

	// Feel free to edit these values
	// to get a different 'look 'n feel'
	DrawCharEmptyField = ' ';
	DrawCharBlockedField = ' ';
	DrawCharFallingField = ' ';

	// Default rotation direction
	RotateClockwise = false;

	// The number of different existing blocks
	BlockCount = 7;

	// Default orientation when the blocks enter
	DefaultBlockOrientation = Up;

	// Draw debug output?
	DebugOutput = false;
 
    // The position of the window
    WindowX = 30;
    WindowY = 1;

implementation

end.

